<?PHP
/*****************************************************************************/
/*                                                                           */
/* Ansilove/PHP 1.00 (c) by Frederic Cambus 2003-2006                        */
/* http://ansilove.sourceforge.net                                           */
/*                                                                           */
/* Created:      2003/07/17                                                  */
/* Last Updated: 2006/01/27                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify      */
/* it under the terms of the GNU General Public License as published by      */
/* the Free Software Foundation; either version 2 of the License, or         */
/* (at your option) any later version.                                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/*****************************************************************************/



/*****************************************************************************/
/* CHECK LIBRARIES                                                           */
/*****************************************************************************/

function check_libraries()
{
   if (!extension_loaded('gd'))
   {
      echo "ERROR: Ansilove requires GD library.\n\n";
      exit(-1);
   }
}



/*****************************************************************************/
/* LOAD BINARY                                                               */
/*****************************************************************************/

function load_binary($input,$output,$columns,$font,$bits,$icecolors)
{
   check_libraries();

/*****************************************************************************/
/* CHECK PARAMETERS AND FORCE DEFAULT VALUES IF INVALID INPUT IS DETECTED    */
/*****************************************************************************/

   if ($columns==0)
   {
      $columns=160;
   }

   if ($bits!=8 && $bits!=9)
   {
      $bits=8;
   }

   switch($font)
   {
   case '80x25':
      $font_file='ansilove_font_pc_80x25.png';
      $font_size_x=9;
      $font_size_y=16;
      break;

   case '80x50':
      $font_file='ansilove_font_pc_80x50.png';
      $font_size_x=9;
      $font_size_y=8;
      break;

   case 'amiga':
      $font_file='ansilove_font_topaz+.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'b-strict':
      $font_file='ansilove_font_b-strict.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'b-struct':
      $font_file='ansilove_font_b-struct.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'microknight':
      $font_file='ansilove_font_microknight.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'mosoul':
      $font_file='ansilove_font_mosoul.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'pot-noodle':
      $font_file='ansilove_font_pot-noodle.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'topaz':
      $font_file='ansilove_font_topaz.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'topaz+':
      $font_file='ansilove_font_topaz+.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   default:
      $font_file='ansilove_font_pc_80x25.png';
      $font_size_x=9;
      $font_size_y=16;
   }



/*****************************************************************************/
/* LOAD INPUT FILE                                                           */
/*****************************************************************************/

   if (!$input_file = fopen ($input,'r'))
   {
      echo "ERROR: Can't open file $input\n\n";
      exit(-1);
   }

   $input_file_sauce=load_sauce($input);

   if ($input_file_sauce!=NULL)
   {
      $input_file_size=$input_file_sauce['FileSize'];
   }
   else
   {
      $input_file_size=filesize($input);
   }

   if (!$input_file_buffer = fread($input_file,$input_file_size))
   {
      echo "ERROR: Can't read file $input\n\n";
      exit(-1);
   }

   fclose($input_file);



/*****************************************************************************/
/* LOAD BACKGROUND/FONT AND ALLOCATE IMAGE BUFFER MEMORY                     */
/*****************************************************************************/

   if (!$background = imagecreatefrompng ('fonts/ansilove_background.png'))
   {
      echo "ERROR: Can't open file ansilove_background.png\n\n";
      exit(-1);
   }

   if (!$font = imagecreatefrompng ('fonts/'.$font_file))
   {
      echo "ERROR: Can't open find file $font_file\n\n";
      exit(-1);
   }

   imagecolortransparent($font,20);

   if (!$binary = imagecreate($columns*$bits,(($input_file_size/2)/$columns)*$font_size_y))
   {
      echo "ERROR: Can't allocate buffer image memory\n\n";
      exit(-1);
   }



/*****************************************************************************/
/* PROCESS BINARY                                                            */
/*****************************************************************************/

   imagecolorallocate($binary,0,0,0);

   while ($loop<$input_file_size)
   {
      if ($position_x==$columns)
      {
         $position_x=0;
         $position_y++;
      }

      $character=ord($input_file_buffer[$loop]);
      $attribute=ord($input_file_buffer[$loop+1]);

      $color_background=($attribute & 240)>>4;
      $color_foreground=$attribute & 15;

      if ($color_background>8 && $icecolors==0)
      {
         $color_background-=8;
      }

      imagecopy ($binary, $background, $position_x*$bits, $position_y*$font_size_y, $color_background*9, 0, $bits, $font_size_y);
      imagecopy ($binary, $font, $position_x*$bits, $position_y*$font_size_y, $character*$font_size_x, $color_foreground*$font_size_y, $bits, $font_size_y);

      $position_x++;
      $loop+=2;
   }



/*****************************************************************************/
/* CREATE OUTPUT FILE                                                        */
/*****************************************************************************/

   if ($output=="online")
   {
      Header("Content-type: image/png");
      ImagePNG($binary);
   }
   else
   {
      ImagePNG($binary,$output);
   }



/*****************************************************************************/
/* FREE MEMORY                                                               */
/*****************************************************************************/

   imagedestroy($binary);
   imagedestroy($background);
   imagedestroy($font);
}



/*****************************************************************************/
/* LOAD ADF                                                                  */
/*****************************************************************************/

function load_adf($input,$output)
{
   check_libraries();

/*****************************************************************************/
/* LOAD INPUT FILE                                                           */
/*****************************************************************************/

   if (!$input_file = fopen ($input,'r'))
   {
      echo "ERROR: Can't open file $input\n\n";
      exit(-1);
   }

   $input_file_sauce=load_sauce($input);

   if ($input_file_sauce!=NULL)
   {
      $input_file_size=$input_file_sauce['FileSize'];
   }
   else
   {
      $input_file_size=filesize($input);
   }

   if (!$input_file_buffer = fread($input_file,$input_file_size))
   {
      echo "ERROR: Can't read file $input\n\n";
      exit(-1);
   }

   fclose($input_file);



/*****************************************************************************/
/* ALLOCATE BACKGROUND/FONT IMAGE BUFFER MEMORY                              */
/*****************************************************************************/

   if (!$background = imagecreate(128,16))
   {
      echo "ERROR: Can't allocate background buffer image memory\n\n";
      exit(-1);
   }

   if (!$font = imagecreate(2048,256))
   {
      echo "ERROR: Can't allocate font buffer image memory\n\n";
      exit(-1);
   }

   if (!$font_inverted = imagecreate(2048,16))
   {
      echo "ERROR: Can't allocate temporary font buffer image memory\n\n";
      exit(-1);
   }



/*****************************************************************************/
/* PROCESS ADF PALETTE                                                       */
/*****************************************************************************/

   $adf_colors=array(0,1,2,3,4,5,20,7,56,57,58,59,60,61,62,63);

   for ($loop=0;$loop<16;$loop++)
   {
      $index=($adf_colors[$loop]*3)+1;
      $colors[$loop]=imagecolorallocate($background,ord($input_file_buffer[$index])<<2,ord($input_file_buffer[$index+1])<<2,ord($input_file_buffer[$index+2])<<2);
      $colors[$loop]=imagecolorallocate($font,ord($input_file_buffer[$index])<<2,ord($input_file_buffer[$index+1])<<2,ord($input_file_buffer[$index+2])<<2);
      $colors[$loop]=imagecolorallocate($font_inverted,ord($input_file_buffer[$index])<<2,ord($input_file_buffer[$index+1])<<2,ord($input_file_buffer[$index+2])<<2);
   }

   $colors[16]= imagecolorallocate ($font,0,0,0);
   $colors[20]= imagecolorallocate ($font_inverted,200,220,169);

   for ($loop=0;$loop<16;$loop++)
   {
      imagefilledrectangle($background,$loop<<3,0,($loop<<3)+8,16,$colors[$loop]);
   }



/*****************************************************************************/
/* PROCESS ADF FONT                                                          */
/*****************************************************************************/

   imagefilledrectangle($font_inverted, 0, 0, 2048, 16, $colors[20]);
   imagecolortransparent($font_inverted,$colors[20]);

   for ($loop=0;$loop<256;$loop++)
   {
      for ($adf_font_size_y=0;$adf_font_size_y<16;$adf_font_size_y++)
      {
         $adf_character_line=ord($input_file_buffer[193+$adf_font_size_y+($loop*16)]);

         for ($loop_column=0;$loop_column<8;$loop_column++)
         {
            $adf_character_column=128/pow(2,$loop_column);

            if (($adf_character_line & $adf_character_column)!=$adf_character_column)
            {
               imagesetpixel ($font_inverted,($loop*8)+$loop_column,$adf_font_size_y,$colors[0]);
            }
         }
      }
   }

   for ($loop=1;$loop<16;$loop++)
   {
      imagefilledrectangle($font,0,$loop*16,2048,($loop*16)+16,$colors[$loop]);
   }
   imagefilledrectangle($font,0,0,2048,15,$colors[16]);

   for ($loop=0;$loop<16;$loop++)
   {
      imagecopy ($font, $font_inverted, 0,$loop*16,0,0,2048,16);
   }
   imagecolortransparent($font,$colors[0]);



/*****************************************************************************/
/* ALLOCATE IMAGE BUFFER MEMORY                                              */
/*****************************************************************************/

   if (!$adf = imagecreate(640,((($input_file_size-192-4096-1)/2)/80)*16))
   {
      echo "ERROR: Can't allocate buffer image memory\n\n";
      exit(-1);
   }



/*****************************************************************************/
/* PROCESS ADF                                                               */
/*****************************************************************************/

   imagecolorallocate($adf,0,0,0);

   $loop=192+4096+1;

   while ($loop<$input_file_size)
   {
      if ($position_x==80)
      {
         $position_x=0;
         $position_y++;
      }

      $character=ord($input_file_buffer[$loop]);
      $attribute=ord($input_file_buffer[$loop+1]);

      $color_background=($attribute & 240)>>4;
      $color_foreground=$attribute & 15;

      imagecopy ($adf, $background, $position_x*8, $position_y*16, $color_background*8,0,8,16);
      imagecopy ($adf, $font, $position_x*8, $position_y*16, $character*8, $color_foreground*16,8,16);

      $position_x++;
      $loop+=2;
   }



/*****************************************************************************/
/* CREATE OUTPUT FILE                                                        */
/*****************************************************************************/

   if ($output=="online")
   {
      Header("Content-type: image/png");
      ImagePNG($adf);
   }
   else
   {
      ImagePNG($adf,$output);
   }



/*****************************************************************************/
/* FREE MEMORY                                                               */
/*****************************************************************************/

   imagedestroy($adf);
   imagedestroy($background);
   imagedestroy($font);
   imagedestroy($font_inverted);
}



/*****************************************************************************/
/* LOAD ANSI                                                                 */
/*****************************************************************************/

function load_ansi($input,$output,$font,$bits,$icecolors)
{
   check_libraries();

/*****************************************************************************/
/* CHECK PARAMETERS AND FORCE DEFAULT VALUES IF INVALID INPUT IS DETECTED    */
/*****************************************************************************/

   $columns=80;

   if ($bits!=8 && $bits!=9)
   {
      $bits=8;
   }

   switch($font)
   {
   case '80x25':
      $font_file='ansilove_font_pc_80x25.png';
      $font_size_x=9;
      $font_size_y=16;
      break;

   case '80x50':
      $font_file='ansilove_font_pc_80x50.png';
      $font_size_x=9;
      $font_size_y=8;
      break;

   case 'amiga':
      $font_file='ansilove_font_topaz+.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'b-strict':
      $font_file='ansilove_font_b-strict.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'b-struct':
      $font_file='ansilove_font_b-struct.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'microknight':
      $font_file='ansilove_font_microknight.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'mosoul':
      $font_file='ansilove_font_mosoul.png';
      $font_size_x=8;
      $font_size_y=8;
      $bits=8;
      break;

   case 'pot-noodle':
      $font_file='ansilove_font_pot-noodle.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'topaz':
      $font_file='ansilove_font_topaz.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   case 'topaz+':
      $font_file='ansilove_font_topaz+.png';
      $font_size_x=8;
      $font_size_y=11;
      $bits=8;
      break;

   default:
      $font_file='ansilove_font_pc_80x25.png';
      $font_size_x=9;
      $font_size_y=16;
   }



/*****************************************************************************/
/* LOAD INPUT FILE                                                           */
/*****************************************************************************/

   if (!$input_file = fopen ($input,'r'))
   {
      echo "ERROR: Can't open file $input\n\n";
      exit(-1);
   }

   $input_file_sauce=load_sauce($input);

   if ($input_file_sauce!=NULL)
   {
      $input_file_size=$input_file_sauce['FileSize'];
   }
   else
   {
      $input_file_size=filesize($input);
   }

   if (!$input_file_buffer = fread($input_file,$input_file_size))
   {
      echo "ERROR: Can't read file $input\n\n";
      exit(-1);
   }

   fclose($input_file);



/*****************************************************************************/
/* LOAD BACKGROUND/FONT                                                      */
/*****************************************************************************/

   if (!$background = imagecreatefrompng ('fonts/ansilove_background.png'))
   {
      echo "ERROR: Can't find file ansilove_background.png\n\n";
      exit(-1);
   }

   if (!$font = imagecreatefrompng ('fonts/'.$font_file))
   {
      echo "ERROR: Can't open find file $font_file\n\n";
      exit(-1);
   }

   imagecolortransparent($font,20);



/*****************************************************************************/
/* ALLOCATE BACKGROUND/FOREGROUND COLOR ARRAYS                               */
/*****************************************************************************/

   $background_colors[40]=0;  $background_colors[41]=4;  $background_colors[42]=2;
   $background_colors[43]=6;  $background_colors[44]=1;  $background_colors[45]=5;
   $background_colors[46]=3;  $background_colors[47]=7;

   $foreground_colors[20]=8;  $foreground_colors[21]=12; $foreground_colors[22]=10;
   $foreground_colors[23]=14; $foreground_colors[24]=9;  $foreground_colors[25]=13;
   $foreground_colors[26]=11; $foreground_colors[27]=15;

   $foreground_colors[30]=0;  $foreground_colors[31]=4;  $foreground_colors[32]=2;
   $foreground_colors[33]=6;  $foreground_colors[34]=1;  $foreground_colors[35]=5;
   $foreground_colors[36]=3;  $foreground_colors[37]=7;



/*****************************************************************************/
/* PROCESS ANSI                                                              */
/*****************************************************************************/

   $color_foreground=7;

   while ($loop<$input_file_size)
   {
      $current_character=ord($input_file_buffer[$loop]);
      $next_character=ord($input_file_buffer[$loop+1]);

      if ($position_x==80)
      {
         $position_y++;
         $position_x=0;
      }

/*****************************************************************************/
/* CR+LF                                                                     */
/*****************************************************************************/

      if ($current_character==13)
      {
         if ($next_character==10)
         {
            $position_y++;
            $position_x=0;
            $loop++;
         }
      }

/*****************************************************************************/
/* LF                                                                        */
/*****************************************************************************/

      if ($current_character==10)
      {
         $position_y++;
         $position_x=0;
      }

/*****************************************************************************/
/* TAB                                                                       */
/*****************************************************************************/

      if ($current_character==9)
      {
         $position_x+=8;
      }

/*****************************************************************************/
/* SUB                                                                       */
/*****************************************************************************/

      if ($current_character==26)
      {
         break;
      }



/*****************************************************************************/
/* ANSI SEQUENCE                                                             */
/*****************************************************************************/

      if ($current_character==27 && $next_character==91)
      {
         unset($ansi_sequence);

         for ($ansi_sequence_loop=0;$ansi_sequence_loop<12;$ansi_sequence_loop++)
         {
            $ansi_sequence_character=$input_file_buffer[$loop+2+$ansi_sequence_loop];

/*****************************************************************************/
/* CURSOR POSITION                                                           */
/*****************************************************************************/

            if ($ansi_sequence_character=='H' || $ansi_sequence_character=='f')
            {
               $ansi_sequence_exploded=explode(";",$ansi_sequence);
               $position_y=$ansi_sequence_exploded[0]-1;
               $position_x=$ansi_sequence_exploded[1]-1;

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* CURSOR UP                                                                 */
/*****************************************************************************/

            if ($ansi_sequence_character=='A')
            {
               if ($ansi_sequence=='')
               {
                  $ansi_sequence=1;
               }

               $position_y=$position_y-$ansi_sequence;

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* CURSOR DOWN                                                               */
/*****************************************************************************/

            if ($ansi_sequence_character=='B')
            {
               if ($ansi_sequence=='')
               {
                  $ansi_sequence=1;
               }

               $position_y=$position_y+$ansi_sequence;

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* CURSOR FORWARD                                                            */
/*****************************************************************************/

            if ($ansi_sequence_character=='C')
            {
               if ($ansi_sequence=='')
               {
                  $ansi_sequence=1;
               }

               $position_x=$position_x+$ansi_sequence;
               if ($position_x>80)
               {
                  $position_x=80;
               }

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* CURSOR BACKWARD                                                           */
/*****************************************************************************/

            if ($ansi_sequence_character=='D')
            {
               if ($ansi_sequence=='')
               {
                  $ansi_sequence=1;
               }

               $position_x=$position_x-$ansi_sequence;
               if ($position_x<0)
               {
                  $position_x=0;
               }

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* SAVE CURSOR POSITION                                                      */
/*****************************************************************************/

            if ($ansi_sequence_character=='s')
            {
               $saved_position_y=$position_y;
               $saved_position_x=$position_x;

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* RESTORE CURSOR POSITION                                                   */
/*****************************************************************************/

            if ($ansi_sequence_character=='u')
            {
               $position_y=$saved_position_y;
               $position_x=$saved_position_x;

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* ERASE DISPLAY                                                             */
/*****************************************************************************/

            if ($ansi_sequence_character=='J')
            {
               if ($ansi_sequence==2)
               {
                  unset($ansi_buffer);
               }

               $loop+=$ansi_sequence_loop+2;
               break;
            }

/*****************************************************************************/
/* SET GRAPHIC RENDITION                                                     */
/*****************************************************************************/

            if ($ansi_sequence_character=='m')
            {
               $ansi_sequence_exploded=explode(";",$ansi_sequence);

               for($loop_ansi_sequence=0;$loop_ansi_sequence<sizeof($ansi_sequence_exploded);$loop_ansi_sequence++)
               {
                  $ansi_sequence_value=$ansi_sequence_exploded[$loop_ansi_sequence];

                  if ($ansi_sequence_value==0)
                  {
                     $color_foreground=7;
                     $color_background=0;
                     $bold=0;
                     $blink=0;
                  }

                  if ($ansi_sequence_value==1)
                  {
                     $color_foreground+=8;
                     $bold=1;
                  }

                  if ($ansi_sequence_value==5)
                  {
                     $color_background+=8;
                     $blink=1;
                  }

                  if ($ansi_sequence_value>29 && $ansi_sequence_value<40)
                  {
                     if ($bold==1)
                     {
                        $ansi_sequence_value=$ansi_sequence_value-10;
                     }
                     $color_foreground=$foreground_colors[$ansi_sequence_value];
                  }

                  if ($ansi_sequence_exploded[$loop_ansi_sequence]>39 && $ansi_sequence_exploded[$loop_ansi_sequence]<50)
                  {
                     if ($blink==1 && $icecolors==1)
                     {
                        $ansi_sequence_value=$ansi_sequence_value-20;
                        $color_background=$foreground_colors[$ansi_sequence_value];
                     }
                     else
                     {
                        $color_background=$background_colors[$ansi_sequence_value];
                     }
                  }
               }

               $loop+=$ansi_sequence_loop+2;
               break;
            }
            $ansi_sequence.=$ansi_sequence_character;
         }
      }
      elseif ($current_character!=10 && $current_character!=13 && $current_character!=9)
      {

/*****************************************************************************/
/* RECORD NUMBER OF COLUMNS AND LINES USED                                   */
/*****************************************************************************/

         if ($position_x>$position_x_max)
         {
            $position_x_max=$position_x;
         }

         if ($position_y>$position_y_max)
         {
            $position_y_max=$position_y;
         }



/*****************************************************************************/
/* WRITE CURRENT CHARACTER INFO IN A TEMPORARY ARRAY                         */
/*****************************************************************************/

         $ansi_buffer[]=$position_x;
         $ansi_buffer[]=$position_y;
         $ansi_buffer[]=$color_background;
         $ansi_buffer[]=$color_foreground;
         $ansi_buffer[]=$current_character;

         $position_x++;
      }
      $loop++;
   }



/*****************************************************************************/
/* ALLOCATE IMAGE BUFFER MEMORY                                              */
/*****************************************************************************/

   $position_x_max++;
   $position_y_max++;

   if (strtolower(substr($input,strlen($input)-4,4))=='.diz')
   {
      $columns=$position_x_max;
   }

   if (!$ansi = imagecreate($columns*$bits,($position_y_max)*$font_size_y))
   {
      echo "ERROR: Can't allocate buffer image memory\n\n";
      exit(-1);
   }



/*****************************************************************************/
/* RENDER ANSI                                                               */
/*****************************************************************************/

   imagecolorallocate($ansi,0,0,0);

   for($loop=0;$loop<sizeof($ansi_buffer);$loop+=5)
   {
      $position_x=$ansi_buffer[$loop];
      $position_y=$ansi_buffer[$loop+1];
      $color_background=$ansi_buffer[$loop+2];
      $color_foreground=$ansi_buffer[$loop+3];
      $character=$ansi_buffer[$loop+4];

      imagecopy ($ansi, $background, $position_x*$bits, $position_y*$font_size_y, $color_background*9, 0, $bits, $font_size_y);
      imagecopy ($ansi, $font, $position_x*$bits, $position_y*$font_size_y, $character*$font_size_x, $color_foreground*$font_size_y, $bits, $font_size_y);
   }



/*****************************************************************************/
/* CREATE OUTPUT FILE                                                        */
/*****************************************************************************/

   if ($output=="online")
   {
      Header("Content-type: image/png");
      ImagePNG($ansi);
   }
   else
   {
      ImagePNG($ansi,$output);
   }



/*****************************************************************************/
/* FREE MEMORY                                                               */
/*****************************************************************************/

   imagedestroy($ansi);
   imagedestroy($background);
   imagedestroy($font);
}



/*****************************************************************************/
/* LOAD SAUCE                                                                */
/*****************************************************************************/

function load_sauce($input)
{

/*****************************************************************************/
/* LOAD INPUT FILE                                                           */
/*****************************************************************************/

   if (!$input_file = fopen ($input,r))
   {
      echo "ERROR: Can't open file $input\n\n";
      exit(-1);
   }

   $input_file_size=filesize($input);



/*****************************************************************************/
/* PROCESS SAUCE                                                             */
/*****************************************************************************/

   if($input_file_size>=128)
   {
      fseek($input_file,$input_file_size-128);
      $input_file_buffer=fread($input_file,128);

      $sauce['ID']=substr($input_file_buffer,0,5);
      $sauce['Version']=substr($input_file_buffer,5,2);
      $sauce['Title']=substr($input_file_buffer,7,35);
      $sauce['Author']=substr($input_file_buffer,42,20);
      $sauce['Group']=substr($input_file_buffer,62,20);
      $sauce['Date']=substr($input_file_buffer,82,8);
      $sauce=array_merge($sauce,unpack('lFileSize/CDataType/CFileType/S4TInfo/CComments/CFlags',substr($input_file_buffer,90,16)));
      $sauce['Filler']=substr($input_file_buffer,106,22);
   }



/*****************************************************************************/
/* PROCESS SAUCE COMMENTS                                                    */
/*****************************************************************************/

   $sauce_comment_lines=$sauce['Comments'];

   if ($sauce_comment_lines!=0)
   {
      if($input_file_size>=128+5+($sauce_comment_lines*64))
      {
         fseek($input_file,($input_file_size-128-5-($sauce_comment_lines*64)));
         $comment_id=fread($input_file,5);

         if ($comment_id=='COMNT')
         {
            for ($loop=0;$loop<$sauce_comment_lines;$loop++)
            {
               fseek($input_file,($input_file_size-128-($sauce_comment_lines*64)+($loop*64)));

               $comment_line="Comment_Line_";
               if ($loop<9)
               {
                  $comment_line.="0";
               }
               $comment_line.=($loop+1);

               $sauce[$comment_line]=fread($input_file,64);
            }
         }
      }
   }

   fclose($input_file);



/*****************************************************************************/
/* RETURN SAUCE                                                              */
/*****************************************************************************/

   if ($sauce['ID']=='SAUCE')
   {
      return($sauce);
   }
}
?>